@@ -14,6 +14,9 @@ from src.events.types import (
     LLMCallFinishEvent, ToolExecutionEvent, LLMCallErrorEvent
 )
 from src.events.base import generate_span_id
+from src.utils.logger import get_logger
+
+logger = get_logger(__name__)
 
 
 class AnthropicProvider(BaseProvider):
@@ -74,6 +77,131 @@ class AnthropicProvider(BaseProvider):
         """Check if Anthropic request is for streaming."""
         return body.get("stream", False) is True
     
+    def parse_streaming_response(self, body_bytes: bytes) -> Optional[Dict[str, Any]]:
+        """Parse Anthropic SSE streaming response into structured data.
+        
+        Extracts the message from message_start event, aggregates text from content_block_delta,
+        tool_use blocks from content_block_start, and merges usage from message_delta event.
+        
+        Args:
+            body_bytes: Raw SSE response bytes
+            
+        Returns:
+            Parsed response dict matching Anthropic's non-streaming format, or None if parsing fails
+        """
+        try:
+            import json
+            text = body_bytes.decode('utf-8')
+            lines = text.split('\n')
+            
+            # Parse all SSE events
+            message_data = None
+            usage_data = None
+            content_blocks = {}  # Track content blocks by index
+            
+            current_event = None
+            current_data = []
+            
+            def process_event(event_type: str, event_data: str):
+                """Process a single SSE event."""
+                nonlocal message_data, usage_data
+                
+                try:
+                    parsed = json.loads(event_data)
+                    
+                    # Extract message from message_start
+                    if event_type == 'message_start' and 'message' in parsed:
+                        message_data = parsed['message']
+                    
+                    # Track content blocks as they start
+                    elif event_type == 'content_block_start':
+                        index = parsed.get('index')
+                        content_block = parsed.get('content_block', {})
+                        block_type = content_block.get('type')
+                        
+                        if block_type == 'text':
+                            content_blocks[index] = {'type': 'text', 'text': ''}
+                        elif block_type == 'tool_use':
+                            content_blocks[index] = {
+                                'type': 'tool_use',
+                                'id': content_block.get('id'),
+                                'name': content_block.get('name'),
+                                'input': ''
+                            }
+                    
+                    # Accumulate deltas for each content block
+                    elif event_type == 'content_block_delta':
+                        index = parsed.get('index')
+                        delta = parsed.get('delta', {})
+                        
+                        if index in content_blocks:
+                            if delta.get('type') == 'text_delta':
+                                content_blocks[index]['text'] += delta.get('text', '')
+                            elif delta.get('type') == 'input_json_delta':
+                                content_blocks[index]['input'] += delta.get('partial_json', '')
+                    
+                    # Extract usage from message_delta
+                    elif event_type == 'message_delta' and 'usage' in parsed:
+                        usage_data = parsed['usage']
+                        
+                except json.JSONDecodeError:
+                    pass
+            
+            # Parse SSE format
+            for line in lines:
+                line = line.strip()
+                
+                if line.startswith('event:'):
+                    # Process previous event if complete
+                    if current_event and current_data:
+                        process_event(current_event, ''.join(current_data))
+                    
+                    current_event = line[6:].strip()
+                    current_data = []
+                
+                elif line.startswith('data:'):
+                    current_data.append(line[5:].strip())
+                
+                elif line == '' and current_event and current_data:
+                    # End of event - process it
+                    process_event(current_event, ''.join(current_data))
+                    current_event = None
+                    current_data = []
+            
+            # Build final content array from all blocks
+            if message_data and content_blocks:
+                content_array = []
+                for index in sorted(content_blocks.keys()):
+                    block = content_blocks[index]
+                    if block['type'] == 'text':
+                        content_array.append({'type': 'text', 'text': block['text']})
+                    elif block['type'] == 'tool_use':
+                        # Parse the accumulated JSON input
+                        try:
+                            input_data = json.loads(block['input']) if block['input'] else {}
+                        except json.JSONDecodeError:
+                            input_data = {}
+                        
+                        content_array.append({
+                            'type': 'tool_use',
+                            'id': block['id'],
+                            'name': block['name'],
+                            'input': input_data
+                        })
+                
+                message_data['content'] = content_array
+                
+                if usage_data:
+                    message_data['usage'] = usage_data
+                    
+                return message_data
+            
+            return None
+            
+        except Exception as e:
+            logger.error(f"Error parsing Anthropic SSE response: {e}", exc_info=True)
+            return None
+    
     def extract_conversation_metadata(self, body: Dict[str, Any]) -> Dict[str, Any]:
         """Extract Anthropic-specific metadata."""
         metadata = {}
@@ -124,6 +252,13 @@ class AnthropicProvider(BaseProvider):
         # Default if no system message found
         return "default-system"
     
+    def _extract_tools_for_session(self, body: Dict[str, Any]) -> Optional[List[Dict[str, Any]]]:
+        """Extract tools from request body for session event."""
+        tools = body.get("tools", [])
+        if tools and isinstance(tools, list):
+            return tools
+        return None
+    
     
     def _extract_usage_tokens(self, response_body: Optional[Dict[str, Any]]) -> tuple[Optional[int], Optional[int], Optional[int]]:
         """Extract token usage from response body.
@@ -198,12 +333,20 @@ class AnthropicProvider(BaseProvider):
             span_id = self.generate_new_span_id()
             self.update_session_span_id(session_id, span_id)
             
+            # Extract tools and prompt for session event
+            tools = self._extract_tools_for_session(body)
+            prompt = self._extract_system_prompt(body)
+            
             session_start_event = SessionStartEvent.create(
                 trace_id=trace_id,
                 span_id=span_id,
                 agent_id=agent_id,
                 session_id=session_id,
-                client_type="gateway"
+                client_type="gateway",
+                vendor=self.name,
+                model=session_info.model,
+                tools=tools,
+                prompt=prompt
             )
             events.append(session_start_event)
         
